Принципы реализации
-------------------


Соглашения о модификации исходного кода
=======================================

### Соглашения о наименовании типов данных ###

Библиотека определяет собственные типы данных,
используемые для реализации криптографических и платформонезависимых преобразований.

Все перечислимые типы имеют суффикс `_t`, например, булев тип
определяется следующим образом.


    typedef enum {
      /*! \brief Ложь */
       ak_false,
      /*! \brief Истина */
       ak_true
    } bool_t;
 
 
Сложносоставные типы данных (классы) определяются в виде структуры
 
    typedef struct foo { 
     /*! \brief Данные */
      int data;
     /*! \brief Методы */
      int ()( const char * ) method;
    } *ak_foo;


Указатель на класс (контекст) `foo` имеет префикс `ak_`, т.е. `ak_foo`.

Исключением из данного правила, имеющим исторические корни,
являются целые типы


    ak_uint8, ak_uint32, ak_uint64 и т.п.
    

### Соглашение о наименовании имен функций ###

Каждая из функций библиотеки имеет следующую структуру своего наименования

    тип ak_[объект]_[действие]_[предлог]_[субъект действия]( указатель на объект )

в частности:

 - `ak`: префикс функций библиотеки
 - `объект`: имя, которое описывает объект, над котором выполняется действие
 - `предлог`: направление действия (`от` или `к` объекту) (может отсутствовать),
 - `субъект`: то, на что направлено действие

В качестве первого аргумента функции 
выступает указатель на объект над которым выполняется действие.

Примерами такого наименования могут служить функции

     int ak_blomkey_export_to_file_with_password( ak_blomkey , ... );
     size_t ak_hash_get_block_size( ak_hash );


Основные действия, применимые к объектам библиотеки,
перечислены в следующей таблице.


   Действие   | Описание
  ------------|--------------
   create     | Создание объекта (конструктор). Объект должен быть предварительно размещен в памяти
   new        | Создание объекта и размещение его в оперативной памяти с помощью вызова функции malloc()
   destroy    | Удаление объекта (деструктор).
   delete     | Удаление объекта и удаление его из оперативной памяти с помщью вызова функции free().
   set        | Присвоение объекту некоторого свойства
   get        | Получение некоторого свойства, принадлежащего объекту
   export     | Сохранение объекта и его свойств в контейнере (файле)
   import     | Считывание объекта из контейнера. В процессе считывания происходит создание объекта (действие `create`) и присвоение объекту заданных свойств (действие `set`)
   load       | Считывание объекта из контейнера, включающее в себя создание объекта, размещение его в оперативной памяти и присвоение заданных свойств.


Схематично, сложные действия могут быть определены следующим образом.

   Действие   | Описание
  ------------|--------------
    new       |  malloc + create
    delete    |  destroy + free
    import    |  create + set 
    load      |  malloc + create + set
